import { Injectable } from '@angular/core';
import { assign, cloneDeep } from 'lodash-es';
import { FuseMockApiService, FuseMockApiUtils } from '@fuse/lib/mock-api';
import { services as servicesData } from './data';

@Injectable({
  providedIn: 'root',
})
export class BoardServiceMockApi {
  private _services: any[] = servicesData;

  /**
   * Constructor
   */
  constructor(private _fuseMockApiService: FuseMockApiService) {
    // Register Mock API handlers
    this.registerHandlers();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Register Mock API handlers
   */
  registerHandlers(): void {
    // -----------------------------------------------------------------------------------------------------
    // @ Services - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/board/service/services', 300)
      .reply(({ request }) => {
        // Get available queries
        const search = request.params.get('search');
        const sort = request.params.get('sort') || 'name';
        const order = request.params.get('order') || 'asc';
        const page = parseInt(request.params.get('page') ?? '1', 10);
        const size = parseInt(request.params.get('size') ?? '10', 10);

        // Clone the services
        let services: any[] | null = cloneDeep(this._services);

        // Sort the services
        if (sort === 'sku' || sort === 'name' || sort === 'active') {
          services.sort((a, b) => {
            const fieldA = a[sort].toString().toUpperCase();
            const fieldB = b[sort].toString().toUpperCase();
            return order === 'asc'
              ? fieldA.localeCompare(fieldB)
              : fieldB.localeCompare(fieldA);
          });
        } else {
          services.sort((a, b) =>
            order === 'asc' ? a[sort] - b[sort] : b[sort] - a[sort]
          );
        }

        // If search exists...
        if (search) {
          // Filter the services
          services = services.filter(
            (contact: any) =>
              contact.name &&
              contact.name.toLowerCase().includes(search.toLowerCase())
          );
        }

        // Paginate - Start
        const servicesLength = services.length;

        // Calculate pagination details
        const begin = page * size;
        const end = Math.min(size * (page + 1), servicesLength);
        const lastPage = Math.max(Math.ceil(servicesLength / size), 1);

        // Prepare the pagination object
        let pagination = {};

        // If the requested page number is bigger than
        // the last possible page number, return null for
        // services but also send the last possible page so
        // the app can navigate to there
        if (page > lastPage) {
          services = null;
          pagination = {
            lastPage,
          };
        } else {
          // Paginate the results by size
          services = services.slice(begin, end);

          // Prepare the pagination mock-api
          pagination = {
            length: servicesLength,
            size: size,
            page: page,
            lastPage: lastPage,
            startIndex: begin,
            endIndex: end - 1,
          };
        }

        // Return the response
        return [
          200,
          {
            services,
            pagination,
          },
        ];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Service - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/board/service/service')
      .reply(({ request }) => {
        // Get the id from the params
        const id = request.params.get('id');

        // Clone the services
        const services = cloneDeep(this._services);

        // Find the service
        const service = services.find((item: any) => item.id === id);

        // Return the response
        return [200, service];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Service - POST
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPost('api/apps/board/service/service')
      .reply(() => {
        // Generate a new service
        const newService = {
          id: FuseMockApiUtils.guid(),
          category: '',
          name: 'A New User',
          description: '',
          tags: [],
          sku: '',
          barcode: '',
          brand: '',
          vendor: '',
          stock: '',
          reserved: '',
          cost: '',
          basePrice: '',
          taxPercent: '',
          price: '',
          weight: '',
          thumbnail: '',
          images: [],
          active: false,
        };

        // Unshift the new service
        this._services.unshift(newService);

        // Return the response
        return [200, newService];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Service - PATCH
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPatch('api/apps/board/service/service')
      .reply(({ request }) => {
        // Get the id and service
        const id = request.body.id;
        const service = cloneDeep(request.body.service);

        // Prepare the updated service
        let updatedService = null;

        // Find the service and update it
        this._services.forEach((item, index, services) => {
          if (item.id === id) {
            // Update the service
            services[index] = assign({}, services[index], service);

            // Store the updated service
            updatedService = services[index];
          }
        });

        // Return the response
        return [200, updatedService];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Service - DELETE
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onDelete('api/apps/board/service/service')
      .reply(({ request }) => {
        // Get the id
        const id = request.params.get('id');

        // Find the service and delete it
        this._services.forEach((item, index) => {
          if (item.id === id) {
            this._services.splice(index, 1);
          }
        });

        // Return the response
        return [200, true];
      });
  }
}
